command_failed_codes = {
    0x00: "Invalid instruction",
    0x02: "Component ID invalid",
    0x03: "Page ID invalid",
    0x04: "Picture ID invalid",
    0x05: "Font ID invalid",
    0x11: "Baud rate setting invalid",
    0x12: "Curve control ID number or channel number is invalid",
    0x1A: "Variable name invalid",
    0x1B: "Variable operation invalid",
    0x1C: "Failed to assign",
    0x1D: "Operate EEPROM failed",
    0x1E: "Parameter quantity invalid",
    0x1F: "IO operation failed",
    0x20: "Undefined escape characters",
    0x23: "Too long variable name",
}


class NextionException(Exception):
    pass


class ConnectionFailed(NextionException):
    pass


class CommandFailed(NextionException):
    def __init__(self, command, code):
        if code in command_failed_codes:
            msg = "%s for command: %s" % (command_failed_codes[code], command)
        else:
            msg = "Unknown response code 0x%02x for command: '%s'" % (code, command)

        super(CommandFailed, self).__init__(msg)


class CommandTimeout(NextionException):
    pass



import asyncio
import binascii
import logging
import typing
from enum import IntEnum

logger = logging.getLogger("nextion").getChild(__name__)


class EventType(IntEnum):
    TOUCH = 0x65  # Touch event
    TOUCH_COORDINATE = 0x67  # Touch coordinate
    TOUCH_IN_SLEEP = 0x68  # Touch event in sleep mode
    AUTO_SLEEP = 0x86  # Device automatically enters into sleep mode
    AUTO_WAKE = 0x87  # Device automatically wake up
    STARTUP = 0x88  # System successful start up
    SD_CARD_UPGRADE = 0x89  # Start SD card upgrade


class ResponseType(IntEnum):
    STRING = 0x70
    NUMBER = 0x71
    PAGE = 0x66


class NextionProtocol(asyncio.Protocol):
    EOL = b"\xff\xff\xff"

    def __init__(self, event_message_handler: typing.Callable):
        self.transport = None
        self.buffer = b""
        self.queue = asyncio.Queue()
        self.connect_future = asyncio.get_event_loop().create_future()
        self.event_message_handler = event_message_handler

    def close(self):
        if self.transport:
            self.transport.close()

        if not self.connect_future.done():
            self.connect_future.set_result(False)

    async def wait_connection(self):
        await self.connect_future

    def connection_made(self, transport):
        self.transport = transport
        logger.info("Connected to serial")
        self.connect_future.set_result(True)

    def is_event(self, message):
        return len(message) > 0 and message[0] in EventType.__members__.values()

    def data_received(self, data):
        self.buffer += data

        while True:
            message, eol, leftover = self.buffer.partition(self.EOL)
            if eol == b'':  # EOL not found
                break

            logger.debug("received: %s", binascii.hexlify(message))
            self.buffer = leftover

            if self.is_event(message):
                self.event_message_handler(message)
            else:
                self.queue.put_nowait(message)

    def read_no_wait(self):
        return self.queue.get_nowait()

    async def read(self):
        return await self.queue.get()

    def write(self, data):
        if isinstance(data, str):
            data = data.encode()
        self.transport.write(data + self.EOL)
        logger.debug("sent: %s", data)

    def connection_lost(self, exc):
        logger.error("Connection lost")
        if not self.connect_future.done():
            self.connect_future.set_result(False)
        # self.connect_future = asyncio.get_event_loop().create_future()



import asyncio
import binascii
import logging
import struct
import typing
from collections import namedtuple

import serial_asyncio

from .exceptions import CommandFailed, CommandTimeout, ConnectionFailed
from .protocol import EventType, NextionProtocol, ResponseType

TIME_TO_RECOVER_FROM_SLEEP = 0.15
IO_TIMEOUT = 0.15
BAUDRATES = [2400, 4800, 9600, 19200, 38400, 57600, 115200, 230400]


logger = logging.getLogger("nextion").getChild(__name__)

TouchDataPayload = namedtuple("Touch", "page_id component_id touch_event")
TouchCoordinateDataPayload = namedtuple("TouchCoordinate", "x y touch_event")


class Nextion:
    def __init__(
        self,
        url: str,
        baudrate: int = None,
        event_handler: typing.Callable[[EventType, any], None] = None,
        loop=asyncio.get_event_loop(),
    ):
        self._loop = loop

        self._url = url
        self._baudrate = baudrate
        self._connection = None
        self._command_lock = asyncio.Lock()
        self.event_handler = event_handler or (
            lambda t, d: logger.info("Event %s data: %s" % (t, str(d)))
        )

        self._sleeping = True
        self.sets_todo = {}

    async def on_startup(self):
        await self.command("bkcmd=3")  # Let's ensure we receive expected responses

    async def on_wakeup(self):
        await asyncio.sleep(
            TIME_TO_RECOVER_FROM_SLEEP
        )  # do not execute next messages until full wakeup
        for k, v in self.sets_todo.items():
            self._loop.create_task(self.set(k, v))
        self.sets_todo = {}
        self._sleeping = False

    def event_message_handler(self, message):
        logger.debug("Handle event: %s", message)

        typ = message[0]
        if typ == EventType.TOUCH:  # Touch event
            self.event_handler(
                EventType(typ),
                TouchDataPayload._make(struct.unpack("BBB", message[1:])),
            )
        elif typ == EventType.TOUCH_COORDINATE:  # Touch coordinate
            self.event_handler(
                EventType(typ),
                TouchCoordinateDataPayload._make(struct.unpack("HHB", message[1:])),
            )
        elif typ == EventType.TOUCH_IN_SLEEP:  # Touch event in sleep mode
            self.event_handler(
                EventType(typ),
                TouchCoordinateDataPayload._make(struct.unpack("HHB", message[1:])),
            )
        elif typ == EventType.AUTO_SLEEP:  # Device automatically enters into sleep mode
            self._sleeping = True
            self.event_handler(EventType(typ), None)
        elif typ == EventType.AUTO_WAKE:  # Device automatically wake up
            self._loop.create_task(self.on_wakeup())
            self.event_handler(EventType(typ), None)
        elif typ == EventType.STARTUP:  # System successful start up
            self._loop.create_task(self.on_startup())
            self.event_handler(EventType(typ), None)
        elif typ == EventType.SD_CARD_UPGRADE:  # Start SD card upgrade
            self.event_handler(EventType(typ), None)
        else:
            logger.warning("Other event: 0x%02x", typ)

    def _make_protocol(self) -> NextionProtocol:
        return NextionProtocol(event_message_handler=self.event_message_handler)

    async def connect(self) -> bool:
        connected = False
        baudrates = BAUDRATES.copy()
        if self._baudrate:
            try:
                baudrates.remove(self._baudrate)
            except ValueError:
                pass
            baudrates.insert(0, self._baudrate)

        for baud in baudrates:
            logger.info("Connecting: %s, baud: %s", self._url, baud)
            try:
                _, self._connection = await serial_asyncio.create_serial_connection(
                    self._loop, self._make_protocol, url=self._url, baudrate=baud
                )
            except OSError as e:
                if e.errno == 2:
                    raise ConnectionFailed("Connect failed: %s" % e)
                else:
                    logger.warning("Baud %s not supported: %s", baud, e)
                    continue

            await self._connection.wait_connection()

            self._connection.write("")

            async with self._command_lock:
                self._connection.write("connect")
                try:
                    result = await self._read()
                    if result[:6] == b"comok ":
                        self._baudrate = baud
                        connected = True
                        break
                    else:
                        logger.warning(
                            "Wrong reply to connect attempt. Closing connection"
                        )
                        self._connection.close()
                except asyncio.TimeoutError as e:
                    logger.warning("Time outed connection attempt. Closing connection")
                    self._connection.close()

            await asyncio.sleep(IO_TIMEOUT)

        if not connected:
            raise ConnectionFailed("Connect failed") from e

        data = result[7:].decode().split(",")
        logger.info("Detected model: %s", data[2])
        logger.info("Firmware version: %s", data[3])
        logger.info("Serial number: %s", data[5])
        logger.debug("Flash size: %s", data[6])

        try:
            await self.command("bkcmd=3")
        except CommandTimeout as e:
            logging.debug('Command "bkcmd=3" timeout')
        self._sleeping = await self.get("sleep")

        logger.info("Successfully connected to the device")
        return True

    async def _read(self, timeout=IO_TIMEOUT):
        return await asyncio.wait_for(self._connection.read(), timeout=timeout)

    async def get(self, key, timeout=IO_TIMEOUT):
        return await self.command("get %s" % key, timeout=timeout)

    async def set(self, key, value, timeout=IO_TIMEOUT):
        if isinstance(value, str):
            out_value = '"%s"' % value
        elif isinstance(value, float):
            logger.warning("Float is not supported. Converting to string")
            out_value = '"%s"' % str(value)
        elif isinstance(value, int):
            out_value = str(value)
        else:
            raise AssertionError(
                'value type "%s" is not supported for set' % type(value).__name__
            )

        if self._sleeping and key not in ["sleep"]:
            logging.debug(
                'Device sleeps. Scheduling "%s" set for execution after wakeup', key
            )
            self.sets_todo[key] = value
        else:
            return await self.command("%s=%s" % (key, out_value), timeout=timeout)

    async def command(self, command, timeout=IO_TIMEOUT):
        async with self._command_lock:
            try:
                while True:
                    logger.debug(
                        "Dropping dangling: %s", self._connection.read_no_wait()
                    )
            except asyncio.QueueEmpty:
                pass

            self._connection.write(command)

            result = None
            data = None
            finished = False

            while not finished:
                try:
                    response = await self._read(timeout=timeout)
                except asyncio.TimeoutError as e:
                    raise CommandTimeout(
                        'Command "%s" response was not received' % command
                    ) from e

                res_len = len(response)
                if res_len == 0:
                    finished = True
                elif res_len == 1:  # is response code
                    response_code = response[0]
                    if response_code == 0x01:  # success
                        result = True
                        finished = True
                    else:
                        raise CommandFailed(command, response_code)
                else:
                    type_ = response[0]
                    raw = response[1:]
                    if type_ == ResponseType.PAGE:  # Page ID
                        data = raw[1]
                    elif type_ == ResponseType.STRING:  # string
                        data = raw.decode()
                    elif type_ == ResponseType.NUMBER:  # number
                        data = struct.unpack("i", raw)[0]
                    else:
                        logger.error(
                            "Unknown data received: %s" % binascii.hexlify(response)
                        )

            return data if data is not None else result

    async def sleep(self):
        if self._sleeping:
            return
        await self.set("sleep", 1)
        self._sleeping = True

    async def wakeup(self):
        if not self._sleeping:
            return
        await self.set("sleep", 0)
        await self.on_wakeup()

    async def dim(self, val: int):
        assert 0 <= val <= 100
        await self.set("dim", val)



from os import path as p

from setuptools import find_packages, setup


def read(filename, parent=None):
    parent = parent or __file__

    try:
        with open(p.join(p.dirname(parent), filename)) as f:
            return f.read()
    except IOError:
        return ""


setup(
    name="nextion",
    version="1.4.0",
    packages=find_packages(exclude=["tests", "tests.*"]),
    python_requires=">=3.5.1, <4",
    license="LGPL 3",
    author="Jevgeni Kiski",
    author_email="yozik04@gmail.com",
    description="Nextion display serial client",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    keywords="nextion serial async asyncio",
    url="https://github.com/yozik04/nextion",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
    ],
    install_requires=["pyserial-asyncio"],
    setup_requires=["wheel"],
    tests_require=["asynctest", "mock"],
)
